from minesweeperBoard import Board
from minesweeperGraphics import * # pylint: disable=W0614

import sys
from stddraw import show
import math

class MinesweeperGame():
    def __init__(self, x, y, d):
        self.board = Board(x, y, d)
        drawBoard(x, y)
        self.gameover = False
        self.gamewon = False

    def _reveal(self, x, y, first=True):
        # Tile has already been checked (Recursion base condition)
        if (self.board.isChecked(x, y)):
            return

        drawReveal(x, y)

        # Clicked on a mine. Game Over.
        if first and self.board.isMine(x, y):
            drawMine(x, y)
            self.gameover = True
            return

        # Not a mine and not checked before. Mark as checked then find num of touching mines
        self.board.checkTile(x, y)
        num = self.board.numOfAdjMines(x, y)
        if num > 0: # Don't draw anything if no touching mines
            drawNum(x, y, num)
            return

        # If a tile is not a mine and hasn't been checked yet, add to a list of tiles to check.
        tilesToCheck = []

        for i in range(-1, 2): # x-1, x, x+1
            for j in range(-1, 2): # y-1, y, y+1
                # don't check center tile. Add only if tile isn't a mine
                if  not (i==0 and j==0) and not self.board.isMine(x+i, y+j): 
                    tilesToCheck.append([x+i, y+j])

        # Check the tiles via recursion.
        for t in tilesToCheck:
            self._reveal(t[0], t[1])

    # Play the game
    def play(self):
        while not self.gameover and not self.gamewon:
            if mousePressed():
                mx = math.floor(mouseX())
                my = math.floor(mouseY())
                
                self._reveal(mx, my)
                if self.board.gameWon():
                    self.gamewon = True
            
            show(0)
            
        if self.gamewon:
            drawWinner(self.board.x, self.board.y)
        elif self.gameover:
            drawGameOver(self.board.x, self.board.y)
        show()

x = int(sys.argv[1])
y = int(sys.argv[2])
d = int(sys.argv[3])
m = MinesweeperGame(x, y, d)
m.play()